# distutils: language=c++
from logrank_python cimport logrank_instance
from logrank_python cimport cpu_parallel_logrank
#import scipy.stats as sp
from libcpp.vector cimport vector

def logrank_test(vector[double] groupa, vector[double] groupb, vector[char] groupacensored, vector[char] groupbcensored):
    return logrank_instance(groupa, groupacensored, groupb, groupbcensored)
    #return 1 - sp.chi2.cdf(logrank, df=1)
	
def multi_logrank_test(vector[vector[double]] groupas, vector[vector[double]] groupbs, vector[vector[char]] groupacensoreds, vector[vector[char]] groupbcensoreds, **kwargs):
    cdef vector[double] testvalues
    if "threadnumber" in kwargs:
        return cpu_parallel_logrank(groupas, groupacensoreds, groupbs, groupbcensoreds, kwargs.get("threadnumber"))
    else:
        return cpu_parallel_logrank(groupas, groupacensoreds, groupbs, groupbcensoreds)
    #cdef vector[double] pvalues
    #cdef unsigned long long i
    #for i in range(testvalues.size()):
     #   pvalues.push_back(1 - sp.chi2.cdf(testvalues[i], df=1))
    #return pvalues
	


import scipy.stats as sp

def logrank_test(groupa, groupb, groupacensored, groupbcensored):
    groupaboth = []
    for i in range(len(groupa)):
        groupaboth.append((groupa[i], groupacensored[i]))
    groupbboth = []
    for i in range(len(groupb)):
        groupbboth.append((groupb[i], groupbcensored[i]))
    groupaboth.sort()
    groupbboth.sort()
    na = len(groupaboth)
    nb = len(groupbboth)
    nasize = na
    nbsize = nb
    n = na + nb
    oa = 0.0
    ea = 0.0
    # ob=0.0
    # eb=0.0
    var = 0.0
    indexa = 0
    indexb = 0
    while na > 0 and nb > 0:
        value = groupaboth[indexa][0]
        count = 0
        countb = 0
        minusna = 0
        minusnb = 0
        if indexb < nbsize and groupbboth[indexb][0] < value:
            minusnb += 1
            value = groupbboth[indexb][0]
            if groupbboth[indexb][1] == 1:
                countb += 1
            indexb += 1
            while indexb < nbsize and value == groupbboth[indexb][0]:
                if groupbboth[indexb][1] == 1:
                    countb += 1
                indexb += 1
                minusnb += 1
        elif indexb < nbsize and groupbboth[indexb][0] == value:
            minusna += 1
            if groupaboth[indexa][1] == 1:
                count += 1
            indexa += 1
            while indexa < nasize and value == groupaboth[indexa][0]:
                if groupaboth[indexa][1] == 1:
                    count += 1
                indexa += 1
                minusna += 1
            while indexb < nbsize and value == groupbboth[indexb][0]:
                if groupbboth[indexb][1] == 1:
                    countb += 1
                indexb += 1
                minusnb += 1
        else:
            minusna += 1
            if indexa < nasize and groupaboth[indexa][1] == 1:
                count += 1
            indexa += 1
            while indexa < nasize and value == groupaboth[indexa][0]:
                if groupaboth[indexa][1] == 1:
                    count += 1
                indexa += 1
                minusna += 1
        ea += (float(na) * (float((count + countb)) / float(n)))
        # eb+=(float(nb)*(float((count+countb))/float(n)))
        # ob+=countb
        oa += count
        var += float((na * nb * (count + countb) * (n - (count + countb)))) / float((n * n * (n - 1)))
        na -= minusna
        nb -= minusnb
        n = na + nb
    logrank = ((oa - ea) * (oa - ea)) / var
    pvalue = 1 - sp.chi2.cdf(logrank, df=1)
    return pvalue



from libcpp.vector cimport vector
cdef extern from "logrank_python.cpp":
    double logrank_instance(vector[double] groupa, vector[char] groupacensored, vector[double] groupb, vector[char] groupbcensored)
cdef extern from "logrank_python.cpp":
    vector[double] cpu_parallel_logrank(vector[vector[double]] groupas, vector[vector[char]] groupacensoreds, vector[vector[double]] groupbs, vector[vector[char]] groupbcensoreds)
cdef extern from "logrank_python.cpp":
    vector[double] cpu_parallel_logrank(vector[vector[double]] groupas, vector[vector[char]] groupacensoreds, vector[vector[double]] groupbs, vector[vector[char]] groupbcensoreds, unsigned threadnumber)


# -*- coding: utf-8 -*-

from __future__ import (print_function, absolute_import,
                        division, unicode_literals)

__author__ = 'Andreas Stelzer'
__email__ = 'ga63nep@mytum.de'
__version__ = '0.1.0'

from .fastlogranktest import logrank_test, multi_logrank_test



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from distutils.extension import Extension
from Cython.Build import cythonize

import numpy


source_files = ["fastlogranktest/fastlogranktest.pyx"]
include_dirs = [numpy.get_include()]
extensions = [Extension("fastlogranktest.fastlogranktest",
                        sources=source_files,
                        include_dirs=include_dirs)]
                        
with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = ['cython', 'numpy']


setup(
    name='fastlogranktest',
    version='0.1.1',
    description="Perform the Log-Rank-Test very fast",
    long_description=readme + '\n\n' + history,
    author="Andreas Stelzer",
    author_email='ga63nep@mytum.de',
    url='https://github.com/ga63nep/fastlogranktest',
    packages=[
        'fastlogranktest',
    ],
    package_dir={'fastlogranktest':
                 'fastlogranktest'},
    package_data={'': ['*.pyx', '*.pxd', '*.h', '*.txt', '*.dat', '*.csv']},
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='fastlogranktest',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Cython',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    ext_modules=cythonize(extensions),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_fastlogranktest
----------------------------------

Tests for `fastlogranktest` module.
"""

import fastlogranktest
import logrank_function
import random

t1s = []
t2s = []
e1s = []
e2s = []
inputstring = ""
referencepvalues = []
cythoncpppvalues = []
cythonmulticpppvalues = []
cythoncpperrors = 0
cythonmulticpperrors = 0
countoftests = 1000
setsize = 1000
minvalue = 0
maxvalue = 100
survivalinformation = []
censoredinformation = []

print("generate inputs")
# generate survival times
for i in range(setsize):
    value = random.uniform(minvalue, maxvalue)
    survivalinformation.append(value)
    inputstring += (str(value))
    if i < (setsize - 1):
        inputstring += ("\t")
inputstring += ("\n")
# generate censored information
for i in range(setsize):
    value = random.randint(0, 1)
    censoredinformation.append(value)
    inputstring += (str(value))
    if i < (setsize - 1):
        inputstring += ("\t")
inputstring += ("\n")
# generate tests
for i in range(countoftests):
    t1 = []
    e1 = []
    t2 = []
    e2 = []
    for j in range(setsize):
        value = random.randint(1, 2)
        if value == 1:
            t1.append(survivalinformation[j])
            e1.append(censoredinformation[j])
        elif value == 2:
            t2.append(survivalinformation[j])
            e2.append(censoredinformation[j])
        inputstring += (str(value))
        if j < (setsize - 1):
            inputstring += ("\t")
    t1s.append(t1)
    t2s.append(t2)
    e1s.append(e1)
    e2s.append(e2)
    inputstring += ("\n")

print("calculate with python")
# python
for i in range(countoftests):
    referencepvalues.append(logrank_function.logrank_test(t1s[i], t2s[i], e1s[i], e2s[i]))

print("calculate with cython(cpp)")
# cytoncpp
for i in range(countoftests):
    cythoncpppvalues.append(fastlogranktest.logrank_test(t1s[i], t2s[i], e1s[i], e2s[i]))

print("calculate with cython(cpp)(multithreaded)")
# cythonmulticpp
cythonmulticpppvalues = fastlogranktest.multi_logrank_test(t1s, t2s, e1s, e2s)

print("check everything\n")
# checks
for i in range(countoftests):
    if abs(referencepvalues[i] - cythoncpppvalues[i]) > 0.00000001:
        print(referencepvalues[i])
        print(cythoncpppvalues[i])
        cythoncpperrors += 1
    if abs(referencepvalues[i] - cythonmulticpppvalues[i]) > 0.00000001:
        print(referencepvalues[i])
        print(cythonmulticpppvalues[i])
        cythonmulticpperrors += 1

if (cythoncpperrors + cythonmulticpperrors) == 0:
    print("0 errors occured")
else:
    errorstr = "cython(own cpp): " + str(cythoncpperrors) + " errors\ncython(multi cpp): " + str(
        cythonmulticpperrors) + " errors"
    print(errorstr)
    exit(-1)
